package com.supervision.knowsub.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.supervision.knowsub.model.SystemFlowRule;
import com.supervision.knowsub.model.SystemFlowRuleUser;
import com.supervision.knowsub.service.FlowRuleManageService;
import com.supervision.knowsub.service.SystemFlowRuleService;
import com.supervision.knowsub.service.SystemFlowRuleUserService;
import com.supervision.knowsub.vo.flow.NodeInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.sql.Struct;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class FlowRuleManageServiceImpl implements FlowRuleManageService {

    private final SystemFlowRuleService systemFlowRuleService;

    private final SystemFlowRuleUserService systemFlowRuleUserService;


    @Override
    public boolean saveFlowRule(String flowId, List<NodeInfo> nodeInfoList) {
        Assert.notEmpty(flowId, "流程id不能为空");
        Assert.notEmpty(nodeInfoList, "节点信息不能为空");

        int i = 0;
        for (NodeInfo nodeInfo : nodeInfoList) {
            SystemFlowRule flowRule = new SystemFlowRule();
            flowRule.setFlowId(flowId);
            flowRule.setRuleName(nodeInfo.getNodeName());
            flowRule.setRuleOrder(i++);
            systemFlowRuleService.save(flowRule);

            systemFlowRuleUserService.saveBatch(nodeInfo.getUserIdList().stream().map(userId -> {
                SystemFlowRuleUser flowRuleUser = new SystemFlowRuleUser();
                flowRuleUser.setFlowId(flowId);
                flowRuleUser.setRuleId(flowRule.getId());
                flowRuleUser.setUserId(userId);
                return flowRuleUser;
            }).toList());
        }


        return true;
    }

    @Override
    public List<NodeInfo> listFlowRule(String flowId) {
        if (StrUtil.isEmpty(flowId)){
            return CollUtil.newArrayList();
        }

        List<SystemFlowRule> flowRuleList = systemFlowRuleService.lambdaQuery().eq(SystemFlowRule::getFlowId, flowId).list();
        if (CollUtil.isNotEmpty(flowRuleList)){
            return CollUtil.newArrayList();
        }
        List<NodeInfo> nodeInfoList = flowRuleList.stream().map(flowRule -> NodeInfo.builder()
                .NodeName(flowRule.getRuleName()).id(flowRule.getId())
                .order(flowRule.getRuleOrder()).build()).toList();

        List<SystemFlowRuleUser> ruleUserList = systemFlowRuleUserService.lambdaQuery().in(SystemFlowRuleUser::getFlowId, flowId).list();
        Map<String, List<SystemFlowRuleUser>> ruleUserGroupByRuleId = ruleUserList.stream().collect(Collectors.groupingBy(SystemFlowRuleUser::getRuleId));
        nodeInfoList.forEach(nodeInfo -> {
            List<SystemFlowRuleUser> ruleUserListByRuleId = ruleUserGroupByRuleId.get(nodeInfo.getId());
            if (CollUtil.isNotEmpty(ruleUserListByRuleId)){
                nodeInfo.setUserIdList(ruleUserListByRuleId.stream().map(SystemFlowRuleUser::getUserId).toList());
            }
        });
        return nodeInfoList;
    }
}
